@@ -61,6 +61,9 @@ GType xfdashboard_windows_view_get_type(void) G_GNUC_CONST;
 gfloat xfdashboard_windows_view_get_spacing(XfdashboardWindowsView *self);
 void xfdashboard_windows_view_set_spacing(XfdashboardWindowsView *self, const gfloat inSpacing);
 
+gboolean xfdashboard_windows_view_get_prevent_upscaling(XfdashboardWindowsView *self);
+void xfdashboard_windows_view_set_prevent_upscaling(XfdashboardWindowsView *self, gboolean inPreventUpscaling);
+
 G_END_DECLS
 
 #endif	/* __XFDASHBOARD_WINDOWS_VIEW__ */
